#!/usr/bin/env node
'use strict';

var _introspectionQuery = require('graphql/utilities/introspectionQuery');

var _buildClientSchema = require('graphql/utilities/buildClientSchema');

var _findBreakingChanges = require('graphql/utilities/findBreakingChanges');

var _graphql = require('graphql');

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

var _cliTable = require('cli-table');

var _cliTable2 = _interopRequireDefault(_cliTable);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var usage = '  Usage: graphql-contract-test ENDPOINT_URL <client_schema_file>';
var error = _chalk2.default.bold.red;
var success = _chalk2.default.bold.green;

async function main() {
  var argv = (0, _minimist2.default)(process.argv.slice(2));

  if (argv._.length < 2) {
    console.log(usage);
    return;
  }

  var endpoint = argv._[0];
  var expectedSchemaFile = argv._[1];

  console.log('[*] Checking ' + endpoint + ' against ' + expectedSchemaFile + '...\n');

  var expectedSchema = void 0;

  _fs2.default.readFile(expectedSchemaFile, 'utf8', function (err, data) {
    if (err) {
      console.log(err);
      process.exit(1);
    }

    expectedSchema = (0, _graphql.buildSchema)(data);
  });

  var response = await (0, _nodeFetch2.default)(endpoint, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({ query: _introspectionQuery.introspectionQuery })
  });

  var _ref = await response.json(),
      data = _ref.data,
      errors = _ref.errors;

  if (errors) {
    throw new Error(JSON.stringify(errors, null, 2));
  }

  var actualSchema = (0, _buildClientSchema.buildClientSchema)(data);
  var breakingChanges = (0, _findBreakingChanges.findBreakingChanges)(expectedSchema, actualSchema);

  if (breakingChanges.length === 0) {
    console.log(success('✨  The server appears to implement the schema you provided'));
    process.exit(0);
  }

  console.log('\uD83D\uDCA9  ' + error('Breaking changes were detected\n'));

  var table = new _cliTable2.default({
    head: ['Change Type', 'Description']
  });

  breakingChanges.forEach(function (change) {
    table.push([change.type, change.description]);
  });

  console.log(table.toString());

  process.exit(1);
}

main().catch(function (e) {
  return console.error(e);
});
